__title__ = '2ip'
__description__ = '2ip.me API Client.'
__url__ = 'https://github.com/python-modules/2ip'
__version__ = '0.0.1'
__author__ = 'gbe0'
__author_email__ = 'python@gbe0.com'
__license__ = 'MIT'
__copyright__ = 'Copyright 2021 gbe0'


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import requests
from ipaddress import ip_address
from typing import Any, Optional

class TwoIP(object):
    """
    2ip.me API client
    """

    # The API URL
    api = 'https://api.2ip.me'

    # List of available API endpoints
    endpoints = {
        'geo': {
            'url': f'{api}/geo.json',
            'description': 'Geographic information',
        },
        'provider': {
            'url': f'{api}/provider.json',
            'description': 'Provider information',
        },
    }

    def __init__(self, key: Optional[str] = None) -> object:
        """Set up new 2ip.me API client.

        Parameters
        ----------
        key : str, optional
            Optional API key to use for requests to the API

        Returns
        -------
        TwoIP : object
            The TwoIP API object

        Examples
        --------
        >>> twoip = TwoIP(key = None)
        """
        # Set API key
        self.__key = key

        # Create empty cache
        self.__cache = {
            'geo': {},
            'provider': {},
            'hosting': {},
        }

        # Debugging
        if key:
            logging.debug(f'Setup of new TwoIP object')
        else:
            logging.debug(f'Setup of new TwoIP object (No API key used - rate limits will apply)')

    @staticmethod
    def __api_request(url: str, params: Optional[dict] = None) -> Any:
        """Send request to the 2ip API and return content.

        Parameters
        ----------
        url : str
            The URL to send request to

        params : dict, Optional
            Optional GET parameters to add to the request

        Returns
        -------
        Any
            The dictionary response

        Raises
        ------
        RuntimeError
            If the API request fails
        """
        # Send the request
        try:
            req = requests.request(method = 'GET', url = url, params = params)
        except Exception as e:
            raise RuntimeError('API request failed') from e

        # Make sure API request didn't result in rate limit
        if req.status_code == 429:
            raise RuntimeError(f'API has reached rate limit; retry in an hour or use an API key')

        # Make sure response code is fine
        if req.status_code != 200:
            raise RuntimeError(f'Received unexpected response code "{req.status_code}" from API')

        # Make sure content type is expected
        if req.headers.get('Content-Type') != 'application/json':
            if req.headers.get('Content-Type'):
                raise RuntimeError(f'Expecting Content-Type header "application/json" but got "{req.headers.get("Content-Type")}"')
            else:
                raise RuntimeError(f'API request did not provide any Content-Type header')

        # Make sure the response can be parsed
        try:
            json = req.json()
        except Exception as e:
            raise RuntimeError('Exception parsing response from API as json') from e

        # Return content
        return json

    def __api_param(self, params: dict) -> dict:
        """Add API key to the list of parameters for the API request.

        Parameters
        ----------
        params : dict
            The parameters to add API key to

        Returns
        -------
        dict
            The built parameter list
        """
        # Add API key if available
        if self.__key:
            params['key'] = self.__key

        # Return params
        return params

    def geo(self, ip: str, force: bool = False, cache: bool = True) -> dict:
        """Perform a Geographic information lookup to the 2ip API.

        By default, lookups will be cached. Subsequent lookups will use the cache (unless disabled) to prevent excessive API requests.

        Parameters
        ----------
        ip : str
            The IP address to lookup

        force : bool, default = False
            Force request to be sent to the API even if the lookup has already been cached

        cache: bool, default = True
            Allow caching of the request (cache can still be bypassed by using the force parameter)

        Returns
        -------
        dict
            The response from API in a dictionary

        Raises
        ------
        ValueError
            Invalid IP address specified

        RuntimeError
            API request failure

        Examples
        --------
        >>> twoip = TwoIP(key = '12345678')
        >>> twoip.geo(ip = '192.0.2.0')
        {'ip': '192.0.2.0', 'country_code': '-', 'country': '-', 'country_rus': 'Неизвестно', 'country_ua': 'Невідомо',
        'region': '-', 'region_rus': 'Неизвестно', 'region_ua': 'Невідомо', 'city': '-', 'city_rus': 'Неизвестно',
        'city_ua': 'Невідомо', 'zip_code': '-', 'time_zone': '-'}
        """
        logging.debug(f'2ip Geo API lookup request for IP "{ip}" (force: {force})')

        # Test if the IP to lookup is an actual IP
        try:
            ip_address(ip)
        except Exception as e:
            logging.error(f'Could not run 2ip Geo API lookup due to invalid IP address "{ip}"')
            raise ValueError(f'Could not validate IP address "{ip}", exception from ipaddress: {e}')

        # If the lookup is cached and this is not a forced lookup, return the cached result
        if ip in self.__cache['geo']:
            logging.debug(f'Cached Geo API entry available for IP "{ip}"')
            if force:
                logging.debug('Cached entry will be ignored for lookup; proceeding with API request')
            else:
                logging.debug('Cached entry will be returned; use the force parameter to force API request')
                return self.__cache['geo'][ip]

        # Add API key to parameters if set
        params = self.__api_param({'ip': ip})

        # Set API url
        url = self.endpoints['geo']['url']

        # Send API request
        try:
            response = self.__api_request(url = url, params = params)
        except Exception as e:
            logging.error('Failed to send API request: {e}')
            raise RuntimeError('Could not run API lookup') from e

        # Cache the response if allowed
        if cache:
            self.__cache['geo'][ip] = response

        # Return response
        return response

    def provider(self, ip: str, force: bool = False, cache: bool = True) -> dict:
        """Perform a provider information lookup to the 2ip API.

        By default, lookups will be cached. Subsequent lookups will use the cache (unless disabled) to prevent excessive API requests.

        Parameters
        ----------
        ip : str
            The IP address to lookup

        force : bool, default = False
            Force request to be sent to the API even if the lookup has already been cached

        cache: bool, default = True
            Allow caching of the request (cache can still be bypassed by using the force parameter)

        Returns
        -------
        dict
            The response from API in a dictionary

        Raises
        ------
        ValueError
            Invalid IP address specified

        RuntimeError
            API request failure

        Examples
        --------
        >>> twoip = TwoIP(key = '12345678')
        >>> twoip.provider(ip = '192.0.2.0')
        {'ip': '192.0.2.0', 'name_ripe': 'Reserved AS', 'name_rus': '', 'ip_range_start': '3221225984', 'ip_range_end': '3221226239',
        'route': '192.0.2.0', 'mask': '24'}
        """
        logging.debug(f'2ip provider API lookup request for IP "{ip}" (force: {force})')

        # Test if the IP to lookup is an actual IP
        try:
            ip_address(ip)
        except Exception as e:
            logging.error(f'Could not run 2ip provider API lookup due to invalid IP address "{ip}"')
            raise ValueError(f'Could not validate IP address "{ip}", exception from ipaddress: {e}')

        # If the lookup is cached and this is not a forced lookup, return the cached result
        if ip in self.__cache['provider']:
            logging.debug(f'Cached provider API entry available for IP "{ip}"')
            if force:
                logging.debug('Cached entry will be ignored for lookup; proceeding with API request')
            else:
                logging.debug('Cached entry will be returned; use the force parameter to force API request')
                return self.__cache['provider'][ip]

        # Add API key to parameters if set
        params = self.__api_param({'ip': ip})

        # Set API url
        url = self.endpoints['provider']['url']

        # Send API request
        try:
            response = self.__api_request(url = url, params = params)
        except Exception as e:
            logging.error('Failed to send API request: {e}')
            raise RuntimeError('Could not run API lookup') from e

        # Cache the response if allowed
        if cache:
            self.__cache['provider'][ip] = response

        # Return response
        return response


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Python API client for 2ip.me

This API client will send requests for various information to 2ip.me and provide a response in either XML or JSON.
"""

# Import logging
import logging
from logging import NullHandler

# Import TwoIP class
from .twoip import TwoIP

# Set default logging handler to avoid "No handler found" warnings.
logging.getLogger(__name__).addHandler(NullHandler())

# Set default logging format
logging.basicConfig(
    format='%(asctime)s,%(msecs)03d %(levelname)-7s [%(filename)s:%(lineno)d] %(message)s',
    datefmt='%Y-%m-%d:%H:%M:%S',
    level=logging.WARN,
)

" Do nothing if running module directly "
if __name__ == '__main__':
  raise RuntimeError('This module contains functions for use in other modules; do not use it directly.')



#!/usr/bin/env python
import os
import sys
from codecs import open
from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

# 'setup.py publish' shortcut.
if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()

about = {}
with open(os.path.join(here, 'twoip', '__version__.py'), 'r', 'utf-8') as f:
    exec(f.read(), about)

with open('README.md', 'r', 'utf-8') as f:
    readme = f.read()

with open('requirements.txt', 'r', 'utf-8') as f:
    requires = f.read().splitlines()

test_requirements = [
    'pytest'
]

setup(
    name=about['__title__'],
    version=about['__version__'],
    description=about['__description__'],
    long_description=readme,
    long_description_content_type='text/markdown',
    author=about['__author__'],
    author_email=about['__author_email__'],
    url=about['__url__'],
    packages=find_packages(exclude=('tests',)),
    include_package_data=True,
    python_requires='>= 3.6',
    install_requires=requires,
    tests_require=test_requirements,
    license=about['__license__'],
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
